#include "EntityOneClickOperator.h"
#include "HBaseView.h"

#include "vlist.h"
#include "BlendRDlg.h"
#include "HSelectionSet.h"
#include "HEventManager.h"

EntityOneClickOperator::EntityOneClickOperator(HBaseView* view, const char* entityType, const char* processType, int DoRepeat, int DoCapture)
	: HOpSelectAperture(view, DoRepeat, DoCapture)
	, m_pCEntityType(entityType)
	, m_pCProcessType(processType)
{
	if (!strcmp(m_pCEntityType, "EDGE"))
	{
		GetView()->SetDynamicHighlighting(false);
		GetView()->SetViewSelectionLevel(HViewSelectionLevel::HSelectionLevelEntity);

		HSelectionSet* selection = GetView()->GetSelection();
		selection->SetSelectionLevel(HSelectLevel::HSelectEntity);
	}
	else if (!strcmp(m_pCEntityType, "BODY"))
	{
		GetView()->SetDynamicHighlighting(true);
		GetView()->SetViewSelectionLevel(HViewSelectionLevel::HSelectionLevelSegment);

		HSelectionSet *highlightSelectionSet = GetView()->GetHighlightSelection();
		highlightSelectionSet->SetSelectionLevel(HSelectLevel::HSelectSegment);
	}
}

EntityOneClickOperator::~EntityOneClickOperator()
{
	GetView()->SetDynamicHighlighting(false);

	HSelectionSet* selection = GetView()->GetSelection();
	selection->Init();
}


// normalize double precision vector in place 
void EntityOneClickOperator::normalize_vector(double* vector)
{
	double length;
	length = sqrt(vector[0] * vector[0] + vector[1] * vector[1] + vector[2] * vector[2]);
	vector[0] /= length;
	vector[1] /= length;
	vector[2] /= length;
}

const char * EntityOneClickOperator::GetName()
{
	return m_pCProcessType;
}

int EntityOneClickOperator::OnLButtonDown(HEventInfo &event)
{
	HPoint  new_pos;
	int		res = 0;

	HSelectionSet* selection = GetView()->GetSelection();

	new_pos = event.GetMouseWindowPos();

	HC_Open_Segment_By_Key(GetView()->GetViewKey());

	res = HC_Compute_Selection(".",
		(selection->GetSubwindowPenetration() ? "" : "./scene/overwrite"),
		"v, selection bias = lines, related selection limit = -1, selection sorting, internal selection limit = 0", new_pos.x, new_pos.y);

	HC_Close_Segment();

	if (0 < res) {
		if (!strcmp(m_pCProcessType, "BLEND"))
		{
			HC_KEY  key = INVALID_KEY;
			HC_Show_Selection_Original_Key(&key);

			PK_ENTITY_t entity = HP_Compute_TagID(key, PK_CLASS_entity);

			PK_CLASS_t ent_class;
			PK_ENTITY_ask_class(entity, &ent_class);

			if (PK_CLASS_edge == ent_class)
			{
				HandleSelection(event);
				GetView()->Update();	// update the scene to reflect the new highlight attributes

				BlendRDlg *pDlg = new BlendRDlg(10.0);
				if (pDlg->DoModal() == IDOK)
				{
					double blendR = pDlg->m_dBlendR;
					((HSampleHModel*)GetView()->GetModel())->BlendEdgeR(entity, blendR);

					GetView()->Update();
				}
			}
		}
		else if (!strcmp(m_pCProcessType, "DELETE_BODY"))
		{
			PK_ENTITY_t* entities;
			int num_entities;
			HP_Compute_Selected_Entity_List(&entities, &num_entities);

			if (0 < num_entities)
			{
				PK_BODY_t body = entities[0];

				((HSampleHModel*)GetView()->GetModel())->DeleteEntity(body);

				GetView()->Update();
			}
		}
	}

	return (HOP_OK);
}